/* -*- C -*-
 * $RCSfile: PgSQL.h,v $
 * $Date: 2006-08-08 10:22:28 -0700 (Tue, 08 Aug 2006) $
 * $Revision: 1.3 $
 * Copyright (C) 2001 Timothy H. Keitt
 */

static SEXP
mkInteger(int val) {
  SEXP out;
  PROTECT(out = allocVector(INTSXP, 1));
  INTEGER(out)[0] = val;
  UNPROTECT(1);
  return(out);
}

static SEXP
mkLogical(int val) {
  SEXP out;
  PROTECT(out = allocVector(LGLSXP, 1));
  LOGICAL(out)[0] = val;
  UNPROTECT(1);
  return(out);
}

static SEXP
mkString_safe(char *string) {
  if (string == NULL) string = "";
  return(mkString(string));
}

static SEXP
fetchPgString(PGresult *result, unsigned row, unsigned col) {

  char *value;

  if (PQgetisnull(result, row, col)) return(NA_STRING);

  value = PQgetvalue(result, row, col);

  return(mkChar(value));

}

static int
fetchPgInteger(PGresult *result, unsigned row, unsigned col) {

  int out;
  char *value;
  SEXP tmp1, tmp2;

  if (PQgetisnull(result, row, col)) return(NA_INTEGER);

  value = PQgetvalue(result, row, col);
 
  PROTECT(tmp1 = allocVector(STRSXP, 1));
  SET_STRING_ELT(tmp1, 0, mkChar(value));
  PROTECT(tmp2 = coerceVector(tmp1, INTSXP));
  out = INTEGER(tmp2)[0];
  UNPROTECT(2);
  return(out);

}

static double
fetchPgDouble(PGresult *result, unsigned row, unsigned col) {

  double out;
  char *value;
  SEXP tmp1, tmp2;

  /* 
   * Bug fix by Dirk Eddelbuettel. NA_INTEGER change to NA_REAL
  */
  if (PQgetisnull(result, row, col)) return(NA_REAL);

  value = PQgetvalue(result, row, col);

  PROTECT(tmp1 = allocVector(STRSXP, 1));
  SET_STRING_ELT(tmp1, 0, mkChar(value));
  PROTECT(tmp2 = coerceVector(tmp1, REALSXP));
  out = REAL(tmp2)[0];
  UNPROTECT(2);
  return(out);

}

static int
fetchPgLogical(PGresult *result, unsigned row, unsigned col) {

  if (PQgetisnull(result, row, col)) return(NA_LOGICAL);

  /*
   * JZ commented out this line and added the next line following
   * a bug report by Mitchell Skinner. 04/13/04
   * if (PQgetvalue(result, row, col) == "t") return(1);
   */
  if (0 == strcmp(PQgetvalue(result, row, col), "t")) return(1);

  return(0);
}

/* 
 * Function declarition
 */

SEXP PgSQLcloseConnection(SEXP);
SEXP PgSQLconnect(SEXP);
SEXP PgSQLconnectionInfo(SEXP);
SEXP PgSQLclearResult(SEXP);
SEXP PgSQLsendQuery(SEXP, SEXP);
SEXP PgSQLresultInfo(SEXP);
SEXP PgSQLcolumnInfo(SEXP);
SEXP PgSQLgetResult(SEXP, SEXP);
